/*
 * Copyright 2013 Mark Slater
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package net.sourceforge.writexml;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.NoSuchElementException;

import static net.sourceforge.writexml.ObjectUtils.checkNotNull;

final class CollectionUtils {

    static <T> Iterable<T> copy(final Iterable<T> source) {
        return new ArrayList<T>() {{
            for (T t : source) {
                add(checkNotNull(t, "Cannot add null to collection."));
            }
        }};
    }

    static <T> Iterable<T> append(final Iterable<T> head, final T tail) {
        checkNotNull(tail, "Cannot append null to an Iterable");
        return new Iterable<T>() {
            @Override
            public Iterator<T> iterator() {
                return new Iterator<T>() {
                    private final Iterator<T> delegate = head.iterator();
                    private boolean hasHadLast = false;

                    @Override
                    public boolean hasNext() {
                        return delegate.hasNext() || !hasHadLast;
                    }

                    @Override
                    public T next() {
                        if (delegate.hasNext()) {
                            return delegate.next();
                        } else {
                            if (hasHadLast) {
                                throw new NoSuchElementException();
                            } else {
                                hasHadLast = true;
                                return tail;
                            }
                        }
                    }

                    @Override
                    public void remove() {
                        throw new UnsupportedOperationException();
                    }
                };
            }
        };
    }
}
